#!/usr/bin/env python3
"""
SSL Checker
SSL/TLS certificate and configuration analyzer.
Parameters:
    - target: Target host or list of hosts
    - port: Port to check (default: 443)
    - timeout: Connection timeout
Requires: cryptography, OpenSSL packages
"""
import socket
import ssl
import argparse
import logging
from datetime import datetime
import json
import threading
import queue
from concurrent.futures import ThreadPoolExecutor
from cryptography import x509
from cryptography.hazmat.backends import default_backend
import OpenSSL.SSL
from urllib.parse import urlparse

class SSLChecker:
    def __init__(self, target, port=443, timeout=5):
        self.target = target
        self.port = port
        self.timeout = timeout
        self.results_queue = queue.Queue()
        self.setup_logging()

    def setup_logging(self):
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s - %(levelname)s - %(message)s'
        )
        self.logger = logging.getLogger(__name__)

    def get_certificate_info(self, cert):
        try:
            cert_info = {
                'subject': dict(x509.load_pem_x509_certificate(
                    cert.public_bytes(OpenSSL.crypto.FILETYPE_PEM),
                    default_backend()
                ).subject.get_attributes()),
                'issuer': dict(x509.load_pem_x509_certificate(
                    cert.public_bytes(OpenSSL.crypto.FILETYPE_PEM),
                    default_backend()
                ).issuer.get_attributes()),
                'version': cert.get_version(),
                'serial_number': cert.get_serial_number(),
                'not_before': cert.get_notBefore().decode(),
                'not_after': cert.get_notAfter().decode(),
                'signature_algorithm': cert.get_signature_algorithm().decode(),
                'extensions': self.get_certificate_extensions(cert),
                'public_key': {
                    'type': cert.get_pubkey().type(),
                    'bits': cert.get_pubkey().bits()
                }
            }
            return cert_info
        except Exception as e:
            self.logger.error(f"Error getting certificate info: {str(e)}")
            return None

    def get_certificate_extensions(self, cert):
        extensions = {}
        for i in range(cert.get_extension_count()):
            ext = cert.get_extension(i)
            extensions[ext.get_short_name().decode()] = str(ext)
        return extensions

    def check_ssl_configuration(self, host):
        try:
            context = ssl.create_default_context()
            context.check_hostname = False
            context.verify_mode = ssl.CERT_NONE
            
            with socket.create_connection((host, self.port), self.timeout) as sock:
                with context.wrap_socket(sock, server_hostname=host) as ssock:
                    return {
                        'version': ssock.version(),
                        'cipher': ssock.cipher(),
                        'compression': ssock.compression(),
                        'alpn_protocol': ssock.selected_alpn_protocol(),
                        'npn_protocol': ssock.selected_npn_protocol()
                    }
        except Exception as e:
            self.logger.error(f"Error checking SSL configuration: {str(e)}")
            return None

    def check_vulnerabilities(self, host):
        vulnerabilities = []
        
        # Check for outdated protocols
        try:
            context = ssl.SSLContext(ssl.PROTOCOL_SSLv23)
            context.verify_mode = ssl.CERT_NONE
            with socket.create_connection((host, self.port), self.timeout) as sock:
                with context.wrap_socket(sock) as ssock:
                    if 'SSLv2' in ssock.version() or 'SSLv3' in ssock.version():
                        vulnerabilities.append({
                            'name': 'Outdated Protocol',
                            'description': f'Server supports {ssock.version()}',
                            'severity': 'High'
                        })
        except:
            pass

        # Check for weak ciphers
        weak_ciphers = ['RC4', 'DES', '3DES', 'MD5']
        try:
            context = ssl.create_default_context()
            context.verify_mode = ssl.CERT_NONE
            with socket.create_connection((host, self.port), self.timeout) as sock:
                with context.wrap_socket(sock, server_hostname=host) as ssock:
                    cipher = ssock.cipher()[0]
                    for weak in weak_ciphers:
                        if weak in cipher:
                            vulnerabilities.append({
                                'name': 'Weak Cipher',
                                'description': f'Server supports {cipher}',
                                'severity': 'Medium'
                            })
        except:
            pass

        return vulnerabilities

    def check_host(self, host):
        try:
            # Parse URL if provided
            parsed = urlparse(host)
            hostname = parsed.hostname or host

            cert = ssl.get_server_certificate((hostname, self.port))
            x509_cert = OpenSSL.crypto.load_certificate(
                OpenSSL.crypto.FILETYPE_PEM, cert)
            
            result = {
                'host': hostname,
                'port': self.port,
                'certificate': self.get_certificate_info(x509_cert),
                'configuration': self.check_ssl_configuration(hostname),
                'vulnerabilities': self.check_vulnerabilities(hostname)
            }
            
            self.results_queue.put(result)
            return True
            
        except Exception as e:
            self.logger.error(f"Error checking {host}: {str(e)}")
            return False

    def check_multiple_hosts(self, hosts, max_threads=10):
        with ThreadPoolExecutor(max_workers=max_threads) as executor:
            executor.map(self.check_host, hosts)

    def save_results(self, output_file):
        results = []
        while not self.results_queue.empty():
            results.append(self.results_queue.get())

        check_results = {
            'timestamp': datetime.now().isoformat(),
            'hosts_checked': len(results),
            'results': results
        }

        try:
            with open(output_file, 'w') as f:
                json.dump(check_results, f, indent=4)
            self.logger.info(f"Results saved to {output_file}")
        except Exception as e:
            self.logger.error(f"Error saving results: {str(e)}")

    def print_summary(self):
        results = []
        while not self.results_queue.empty():
            results.append(self.results_queue.get())

        self.logger.info("\nSSL Check Summary:")
        self.logger.info(f"Total hosts checked: {len(results)}")

        # Count vulnerabilities
        vuln_count = {}
        for result in results:
            for vuln in result.get('vulnerabilities', []):
                vuln_count[vuln['name']] = vuln_count.get(vuln['name'], 0) + 1

        if vuln_count:
            self.logger.info("\nVulnerabilities Found:")
            for vuln, count in vuln_count.items():
                self.logger.info(f"{vuln}: {count} hosts")
        else:
            self.logger.info("\nNo vulnerabilities found")

        # Certificate expiry warning
        for result in results:
            if result.get('certificate'):
                try:
                    not_after = datetime.strptime(
                        result['certificate']['not_after'],
                        '%Y%m%d%H%M%SZ'
                    )
                    days_remaining = (not_after - datetime.now()).days
                    if days_remaining < 30:
                        self.logger.warning(
                            f"Certificate for {result['host']} expires in "
                            f"{days_remaining} days"
                        )
                except:
                    pass

    def start_check(self, output_file=None):
        if isinstance(self.target, list):
            self.check_multiple_hosts(self.target)
        else:
            self.check_host(self.target)
        
        if output_file:
            self.save_results(output_file)
        
        self.print_summary()

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="SSL Checker")
    parser.add_argument("target", help="Target host or list of hosts")
    parser.add_argument("--port", type=int, default=443,
                      help="Port to check")
    parser.add_argument("--timeout", type=int, default=5,
                      help="Connection timeout")
    parser.add_argument("--output", help="Output file for results")
    args = parser.parse_args()

    checker = SSLChecker(args.target, args.port, args.timeout)
    checker.start_check(args.output)
